@@ -1,6 +1,6 @@
 /* The MIT License
 
-   Copyright (c) 2021-2022 Genome Research Ltd.
+   Copyright (c) 2021-2023 Genome Research Ltd.
 
    Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -154,8 +154,19 @@ static void _atomize_allele(abuf_t *buf, bcf1_t *rec, int ial)
                 assert(atom);
                 if ( altb!='-' ) kputc(altb, &atom->alt);
                 if ( refb!='-' ) { kputc(refb, &atom->ref); atom->end++; }
+                continue;
             }
-            else
+            buf->natoms++;
+            hts_expand0(atom_t,buf->natoms,buf->matoms,buf->atoms);
+            atom = &buf->atoms[buf->natoms-1];
+            atom->ref.l = 0;
+            atom->alt.l = 0;
+            kputc(refb, &atom->ref);
+            kputc(altb, &atom->alt);
+            atom->beg = atom->end = i;
+            atom->ial = ial;
+
+            if ( rlen!=alen && (i+1>=rlen || i+1>=alen) )   // the next base is an indel combined with SNV, e.g. C>GGG?
             {
                 buf->natoms++;
                 hts_expand0(atom_t,buf->natoms,buf->matoms,buf->atoms);
@@ -163,13 +174,13 @@ static void _atomize_allele(abuf_t *buf, bcf1_t *rec, int ial)
                 atom->ref.l = 0;
                 atom->alt.l = 0;
                 kputc(refb, &atom->ref);
-                kputc(altb, &atom->alt);
+                kputc(refb, &atom->alt);
                 atom->beg = atom->end = i;
                 atom->ial = ial;
             }
             continue;
         }
-        if ( i+1>=rlen || i+1>=alen )   // is the next base a deletion?
+        if ( i+1>=rlen || i+1>=alen )   // is the next base an indel?
         {
             buf->natoms++;
             hts_expand0(atom_t,buf->natoms,buf->matoms,buf->atoms);
@@ -400,12 +411,12 @@ static void _split_table_set_info(abuf_t *buf, bcf_info_t *info, merge_rule_t mo
             buf->tmp2  = dst.s;
             ret = bcf_update_info(buf->out_hdr, out, tag, buf->tmp2, dst.l, type);
         }
-        if ( ret!=0 ) error("An error occurred while updating INFO/%s\n",tag);
+        if ( ret!=0 ) error("An error occurred while updating INFO/%s (errcode=%d)\n",tag,ret);
     }
 }
 static void _split_table_set_history(abuf_t *buf)
 {
-    int i,j;
+    int i,j,ret;
     bcf1_t *rec = buf->split.rec;
     buf->tmps.l = 0;
     ksprintf(&buf->tmps,"%s|%"PRIhts_pos"|%s|",bcf_seqname(buf->hdr,rec),rec->pos+1,rec->d.allele[0]);
@@ -430,8 +441,8 @@ static void _split_table_set_history(abuf_t *buf)
             kputc(',',&buf->tmps);
         }
         buf->tmps.s[--buf->tmps.l] = 0;
-        if ( (bcf_update_info_string(buf->out_hdr, out, buf->split.info_tag, buf->tmps.s))!=0 )
-            error("An error occurred while updating INFO/%s\n",buf->split.info_tag);
+        if ( (ret=bcf_update_info_string(buf->out_hdr, out, buf->split.info_tag, buf->tmps.s))!=0 )
+            error("An error occurred while updating INFO/%s (errcode=%d)\n",buf->split.info_tag,ret);
     }
 }
 static void _split_table_set_gt(abuf_t *buf)
@@ -657,7 +668,7 @@ static void _split_table_set_format(abuf_t *buf, bcf_fmt_t *fmt, merge_rule_t mo
             #undef BRANCH
             ret = bcf_update_format(buf->out_hdr, out, tag, buf->tmp2, 3*(1+star_allele)*nsmpl, type);
         }
-        if ( ret!=0 ) error("An error occurred while updating FORMAT/%s\n",tag);
+        if ( ret!=0 ) error("An error occurred while updating FORMAT/%s (errcode=%d)\n",tag,ret);
     }
 }
 static inline int _is_acgtn(char *seq)
@@ -742,6 +753,8 @@ void _abuf_split(abuf_t *buf, bcf1_t *rec)
             _split_table_overlap(buf, j, atom);
         }
     }
+    // _split_table_print(buf);
+    // _split_table_print_atoms(buf);
     assert( !buf->rbuf.n ); // all records should be flushed first in the SPLIT mode
 
     // Create the output records, transferring all annotations:
