@@ -1,7 +1,7 @@
 /*  bam2bcf.c -- variant calling.
 
     Copyright (C) 2010-2012 Broad Institute.
-    Copyright (C) 2012-2014 Genome Research Ltd.
+    Copyright (C) 2012-2022 Genome Research Ltd.
 
     Author: Heng Li <lh3@sanger.ac.uk>
 
@@ -40,7 +40,8 @@ extern  void ks_introsort_uint32_t(size_t n, uint32_t a[]);
 
 #define CAP_DIST 25
 
-bcf_callaux_t *bcf_call_init(double theta, int min_baseQ)
+bcf_callaux_t *bcf_call_init(double theta, int min_baseQ, int max_baseQ,
+                             int delta_baseQ)
 {
     bcf_callaux_t *bca;
     if (theta <= 0.) theta = CALL_DEFTHETA;
@@ -48,6 +49,8 @@ bcf_callaux_t *bcf_call_init(double theta, int min_baseQ)
     bca->capQ = 60;
     bca->openQ = 40; bca->extQ = 20; bca->tandemQ = 100;
     bca->min_baseQ = min_baseQ;
+    bca->max_baseQ = max_baseQ;
+    bca->delta_baseQ = delta_baseQ;
     bca->e = errmod_init(1. - theta);
     bca->min_frac = 0.002;
     bca->min_support = 1;
@@ -55,9 +58,13 @@ bcf_callaux_t *bcf_call_init(double theta, int min_baseQ)
     bca->npos = 100;
     bca->ref_pos = (int*) malloc(bca->npos*sizeof(int));
     bca->alt_pos = (int*) malloc(bca->npos*sizeof(int));
+    bca->iref_pos= (int*) malloc(bca->npos*sizeof(int));
+    bca->ialt_pos= (int*) malloc(bca->npos*sizeof(int));
     bca->nqual = 60;
     bca->ref_mq  = (int*) malloc(bca->nqual*sizeof(int));
     bca->alt_mq  = (int*) malloc(bca->nqual*sizeof(int));
+    bca->iref_mq = (int*) malloc(bca->nqual*sizeof(int));
+    bca->ialt_mq = (int*) malloc(bca->nqual*sizeof(int));
     bca->ref_bq  = (int*) malloc(bca->nqual*sizeof(int));
     bca->alt_bq  = (int*) malloc(bca->nqual*sizeof(int));
     bca->fwd_mqs = (int*) malloc(bca->nqual*sizeof(int));
@@ -65,51 +72,123 @@ bcf_callaux_t *bcf_call_init(double theta, int min_baseQ)
     return bca;
 }
 
+void bcf_iaux_destroy(bcf_callaux_t *bca);
 void bcf_call_destroy(bcf_callaux_t *bca)
 {
     if (bca == 0) return;
+    bcf_iaux_destroy(bca);
     errmod_destroy(bca->e);
-    if (bca->npos) { free(bca->ref_pos); free(bca->alt_pos); bca->npos = 0; }
-    free(bca->ref_mq); free(bca->alt_mq); free(bca->ref_bq); free(bca->alt_bq);
+    if (bca->npos) {
+        free(bca->ref_pos);  free(bca->alt_pos);
+        free(bca->iref_pos); free(bca->ialt_pos);
+        bca->npos = 0;
+    }
+    free(bca->ref_mq); free(bca->alt_mq);
+    free(bca->iref_mq); free(bca->ialt_mq);
+    free(bca->ref_bq); free(bca->alt_bq);
     free(bca->fwd_mqs); free(bca->rev_mqs);
     bca->nqual = 0;
     free(bca->bases); free(bca->inscns); free(bca);
 }
 
-// position in the sequence with respect to the aligned part of the read
-static int get_position(const bam_pileup1_t *p, int *len)
+static int get_aux_nm(const bam_pileup1_t *p, int32_t qpos, int is_ref)
 {
-    int icig, n_tot_bases = 0, iread = 0, edist = p->qpos + 1;
-    for (icig=0; icig<p->b->core.n_cigar; icig++)
+    int64_t nm;
+    const bam_pileup_cd *cd = &p->cd;
+
+    if ( PLP_NM(cd) == -1 ) return -1;
+    if ( PLP_NM(cd) == PLP_NM_UNSET )
     {
-        int cig  = bam_get_cigar(p->b)[icig] & BAM_CIGAR_MASK;
-        int ncig = bam_get_cigar(p->b)[icig] >> BAM_CIGAR_SHIFT;
-        if ( cig==BAM_CMATCH || cig==BAM_CEQUAL || cig==BAM_CDIFF )
+        // todo: make this localized to be useful for long reads as well
+        bam1_t *rec = p->b;
+        uint8_t *nm_tag = bam_aux_get(rec, "NM");
+        if ( !nm_tag )
         {
-            n_tot_bases += ncig;
-            iread += ncig;
-            continue;
+            PLP_NM(cd) = -1;
+            return -1;
         }
-        if ( cig==BAM_CINS )
+        nm = bam_aux2i(nm_tag);
+
+        // Count indels as single events, not as the number of inserted/deleted
+        // bases (which is what NM does). Add soft clips as mismatches.
+        int i;
+        for (i=0; i < rec->core.n_cigar; i++)
         {
-            n_tot_bases += ncig;
-            iread += ncig;
-            continue;
+            int val = bam_get_cigar(rec)[i] & BAM_CIGAR_MASK;
+            if ( val==BAM_CSOFT_CLIP )
+            {
+                nm += bam_get_cigar(rec)[i] >> BAM_CIGAR_SHIFT;
+            }
+            else if ( val==BAM_CINS || val==BAM_CDEL )
+            {
+                val = bam_get_cigar(rec)[i] >> BAM_CIGAR_SHIFT;
+                if ( val > 1 ) nm -= val - 1;
+            }
         }
-        if ( cig==BAM_CSOFT_CLIP )
-        {
-            iread += ncig;
-            if ( iread<=p->qpos ) edist -= ncig;
-            continue;
-        }
-        if ( cig==BAM_CDEL ) continue;
-        if ( cig==BAM_CHARD_CLIP ) continue;
-        if ( cig==BAM_CPAD ) continue;
-        if ( cig==BAM_CREF_SKIP ) continue;
-        fprintf(stderr,"todo: cigar %d\n", cig);
-        assert(0);
+        PLP_NM(cd) = nm;
     }
-    *len = n_tot_bases;
+    else
+        nm = PLP_NM(cd);
+
+    // Take into account MNPs, 2% of de novo SNVs appear within 20bp of another de novo SNV
+    //      http://www.genome.org/cgi/doi/10.1101/gr.239756.118
+    nm -= is_ref ? 1 : 2;
+
+    if ( nm < 0 ) nm = 0;
+    if ( nm >= B2B_N_NM ) nm = B2B_N_NM - 1;
+
+    return nm;
+}
+
+// position in the sequence with respect to the aligned part of the read
+static int get_position(const bam_pileup1_t *p, int *len,
+                        int *sc_len, int *sc_dist) {
+    int i, j, edist = p->qpos + 1;
+    int sc_left = 0, sc_right = 0;
+    int sc_left_dist = -1, sc_right_dist = -1;
+
+    // left end
+    for (i = 0; i < p->b->core.n_cigar; i++) {
+        int cig  = bam_get_cigar(p->b)[i] & BAM_CIGAR_MASK;
+        if (cig == BAM_CHARD_CLIP)
+            continue;
+        else if (cig == BAM_CSOFT_CLIP)
+            sc_left += bam_get_cigar(p->b)[i] >> BAM_CIGAR_SHIFT;
+        else
+            break;
+    }
+    if (sc_left)
+        sc_left_dist = p->qpos+1 - sc_left;
+    edist -= sc_left;
+
+    // right end
+    for (j = p->b->core.n_cigar-1; j >= i; j--) {
+        int cig  = bam_get_cigar(p->b)[j] & BAM_CIGAR_MASK;
+        if (cig == BAM_CHARD_CLIP)
+            continue;
+        else if (cig == BAM_CSOFT_CLIP)
+            sc_right += bam_get_cigar(p->b)[j] >> BAM_CIGAR_SHIFT;
+        else
+            break;
+    }
+    if (sc_right)
+        sc_right_dist = p->b->core.l_qseq - sc_right - p->qpos;
+
+    // Distance to nearest soft-clips and length of that clip.
+    if (sc_left_dist >= 0) {
+        if (sc_right_dist < 0 || sc_left_dist < sc_right_dist) {
+            *sc_len  = sc_left;
+            *sc_dist = sc_left_dist;
+        }
+    } else if (sc_right_dist >= 0) {
+        *sc_len  = sc_right;
+        *sc_dist = sc_right_dist;
+    } else {
+        *sc_len  = 0;
+        *sc_dist = 0;
+    }
+
+    *len = p->b->core.l_qseq - sc_left - sc_right;
     return edist;
 }
 
@@ -117,8 +196,12 @@ void bcf_callaux_clean(bcf_callaux_t *bca, bcf_call_t *call)
 {
     memset(bca->ref_pos,0,sizeof(int)*bca->npos);
     memset(bca->alt_pos,0,sizeof(int)*bca->npos);
+    memset(bca->iref_pos,0,sizeof(int)*bca->npos);
+    memset(bca->ialt_pos,0,sizeof(int)*bca->npos);
     memset(bca->ref_mq,0,sizeof(int)*bca->nqual);
     memset(bca->alt_mq,0,sizeof(int)*bca->nqual);
+    memset(bca->iref_mq,0,sizeof(int)*bca->nqual);
+    memset(bca->ialt_mq,0,sizeof(int)*bca->nqual);
     memset(bca->ref_bq,0,sizeof(int)*bca->nqual);
     memset(bca->alt_bq,0,sizeof(int)*bca->nqual);
     memset(bca->fwd_mqs,0,sizeof(int)*bca->nqual);
@@ -126,13 +209,32 @@ void bcf_callaux_clean(bcf_callaux_t *bca, bcf_call_t *call)
     if ( call->ADF ) memset(call->ADF,0,sizeof(int32_t)*(call->n+1)*B2B_MAX_ALLELES);
     if ( call->ADR ) memset(call->ADR,0,sizeof(int32_t)*(call->n+1)*B2B_MAX_ALLELES);
     if ( call->SCR ) memset(call->SCR,0,sizeof(*call->SCR)*(call->n+1));
+    if ( call->SCR ) memset(call->SCR,0,sizeof(*call->SCR)*(call->n+1));
+    if ( bca->fmt_flag&B2B_FMT_NMBZ )
+    {
+        memset(call->ref_nm,0,sizeof(*call->ref_nm)*(call->n+1)*B2B_N_NM);
+        memset(call->alt_nm,0,sizeof(*call->alt_nm)*(call->n+1)*B2B_N_NM);
+    }
+    else
+    {
+        memset(call->ref_nm,0,sizeof(*call->ref_nm)*B2B_N_NM);
+        memset(call->alt_nm,0,sizeof(*call->alt_nm)*B2B_N_NM);
+    }
+    memset(call->QS,0,sizeof(*call->QS)*call->n*B2B_MAX_ALLELES);
+    memset(bca->ref_scl,  0, 100*sizeof(int));
+    memset(bca->alt_scl,  0, 100*sizeof(int));
+    memset(bca->iref_scl, 0, 100*sizeof(int));
+    memset(bca->ialt_scl, 0, 100*sizeof(int));
+    int i;
+    for (i=0; i<2; i++) bca->nnm[i] = 0;
+    for (i=0; i<2; i++) bca->nm[i] = 0;
 }
 
 /*
     Notes:
-    - Called from bam_plcmd.c by mpileup. Amongst other things, sets the bcf_callret1_t.qsum frequencies
-        which are carried over via bcf_call_combine and bcf_call2bcf to the output BCF as the QS annotation.
-        Later it's used for multiallelic calling by bcftools -m
+    - Called from bam_plcmd.c by mpileup. Amongst other things, sets the bcf_callret1_t.QS frequencies
+        which are carried over via bcf_call_combine and bcf_call2bcf to the output BCF as the INFO/QS and FMT/QS annotations.
+        Later it's used for multiallelic calling by `call -m`, `call -mG` and `+trio-dnm`.
     - ref_base is the 4-bit representation of the reference base. It is negative if we are looking at an indel.
  */
 /*
@@ -150,7 +252,6 @@ int bcf_call_glfgen(int _n, const bam_pileup1_t *pl, int ref_base, bcf_callaux_t
     // clean from previous run
     r->ori_depth = 0;
     r->mq0 = 0;
-    memset(r->qsum,0,sizeof(float)*4);
     memset(r->anno,0,sizeof(double)*16);
     memset(r->p,0,sizeof(float)*25);
     r->SCR = 0;
@@ -166,32 +267,111 @@ int bcf_call_glfgen(int _n, const bam_pileup1_t *pl, int ref_base, bcf_callaux_t
         kroundup32(bca->max_bases);
         bca->bases = (uint16_t*)realloc(bca->bases, 2 * bca->max_bases);
     }
+
     // fill the bases array
+    double nqual_over_60 = bca->nqual / 60.0;
+    int ADR_ref_missed[4] = {0};
+    int ADF_ref_missed[4] = {0};
     for (i = n = 0; i < _n; ++i) {
         const bam_pileup1_t *p = pl + i;
-        int q, b, mapQ, baseQ, is_diff, min_dist, seqQ;
+        int b;          // the base or indel type
+        int q;          // the base or indel quality used to calculate PL
+        int seqQ;       // used to cap the indel quality given the sequence context
+        int mapQ;       // to cap the quality for low MQ reads
+        int baseQ;      // used only for supporting INFO annotations
+        int is_diff;    // is this base or indel type different from the reference
+        int min_dist;   // distance from the end, used for tail distance bias
+        if ( bca->fmt_flag&(B2B_INFO_SCR|B2B_FMT_SCR) && PLP_HAS_SOFT_CLIP(&p->cd) ) r->SCR++;
         if (p->is_refskip || (p->b->core.flag&BAM_FUNMAP)) continue;
-        if (p->is_del && !is_indel) continue;
+
+        // The meaning of the indel related variables:
+        //  is_indel  .. is this position currently tested for an indel
+        //  p->is_del .. is the current base a deletion in this read (unrelated to the tested indel)
+        //  p->indel  .. is there an indel starting after this position (i.e. does this read have the tested indel)
+        if (p->is_del && !is_indel) continue;   // not testing an indel and the read has a spanning deletion
+
+        int inm = -1;
+
         ++ori_depth;
-        if (is_indel)
+        if (is_indel)   // testing an indel position
         {
-            b     = p->aux>>16&0x3f;
-            baseQ = q = p->aux&0xff;
-            // This read is not counted as indel. Instead of skipping it, treat it as ref. It is
-            // still only an approximation, but gives more accurate AD counts and calls correctly
-            // hets instead of alt-homs in some cases (see test/mpileup/indel-AD.1.sam)
-            if ( q < bca->min_baseQ ) b = 0, q = (int)bam_get_qual(p->b)[p->qpos];
-            seqQ  = p->aux>>8&0xff;
-            is_diff = (b != 0);
+            b = p->aux>>16&0x3f;        // indel type
+            seqQ = q = (p->aux & 0xff); // mp2 + builtin indel-bias
+
+            if ( !bca->indels_v20 )
+            {
+                /*
+                    This heuristics was introduced by e4e161068 and claims to fix #1446. However, we obtain
+                    correct result on the provided test case even when this code is commented out, so this
+                    may not be needed anymore. Leaving it in only for backward compatibility for now.
+                    See mpileup-tests homdel-issue-1446 and CHM1_CHM13_2.45x-1-1701408 which work only when
+                    this code is disabled.
+                */
+                if (p->indel == 0 && (q < _n/2 || _n > 20)) {
+                    // high quality indel calls without p->indel set aren't
+                    // particularly indicative of being a good REF match either,
+                    // at least not in low coverage.  So require solid coverage
+                    // before we start utilising such quals.
+                    b = 0;
+                    q = (int)bam_get_qual(p->b)[p->qpos];
+                    seqQ = (3*seqQ + 2*q)/8;
+                }
+                if (_n > 20 && seqQ > 40) seqQ = 40;
+            }
+
+            is_diff = b ? 1 : 0;
+            if ( bca->fmt_flag&(B2B_FMT_NMBZ|B2B_INFO_NMBZ|B2B_INFO_NM) )
+            {
+                inm = get_aux_nm(p,p->qpos,is_diff?0:1);
+                if ( inm>=0 )
+                {
+                    bca->nnm[is_diff]++;
+                    bca->nm[is_diff] += inm;
+                }
+            }
+
+            if (q < bca->min_baseQ)
+            {
+                if (!p->indel && b < 4) // not an indel read
+                {
+                    if (bam_is_rev(p->b))
+                        ADR_ref_missed[b]++;
+                    else
+                        ADF_ref_missed[b]++;
+                }
+                continue;
+            }
+            baseQ  = p->aux>>8&0xff;
         }
         else
         {
             b = bam_seqi(bam_get_seq(p->b), p->qpos); // base
             b = seq_nt16_int[b? b : ref_base]; // b is the 2-bit base
-            baseQ = q = (int)bam_get_qual(p->b)[p->qpos];
+
+            // Lowest of this and neighbour quality values
+            uint8_t *qual = bam_get_qual(p->b);
+            q = qual[p->qpos];
+            if (p->qpos > 0 &&
+                q > qual[p->qpos-1]+bca->delta_baseQ)
+                q = qual[p->qpos-1]+bca->delta_baseQ;
+            if (p->qpos+1 < p->b->core.l_qseq &&
+                q > qual[p->qpos+1]+bca->delta_baseQ)
+                q = qual[p->qpos+1]+bca->delta_baseQ;
+
             if (q < bca->min_baseQ) continue;
+            if (q > bca->max_baseQ) q = bca->max_baseQ;
+            baseQ = q;
             seqQ  = 99;
             is_diff = (ref4 < 4 && b == ref4)? 0 : 1;
+            if ( bca->fmt_flag&(B2B_FMT_NMBZ|B2B_INFO_NMBZ|B2B_INFO_NM) )
+            {
+                inm = get_aux_nm(p,p->qpos,is_diff?0:1);
+                if ( inm>=0 )
+                {
+                    bca->nnm[is_diff]++;
+                    bca->nm[is_diff] += inm;
+                }
+            }
         }
         mapQ  = p->b->core.qual < 255? p->b->core.qual : DEF_MAPQ; // special case for mapQ==255
         if ( !mapQ ) r->mq0++;
@@ -201,11 +381,12 @@ int bcf_call_glfgen(int _n, const bam_pileup1_t *pl, int ref_base, bcf_callaux_t
         if (q > 63) q = 63;
         if (q < 4) q = 4;       // MQ=0 reads count as BQ=4
         bca->bases[n++] = q<<5 | (int)bam_is_rev(p->b)<<4 | b;
-        if ( bca->fmt_flag&(B2B_INFO_SCR|B2B_FMT_SCR) && PLP_HAS_SOFT_CLIP(p->cd.i) ) r->SCR++;
+        //if (is_indel) fprintf(stderr,"xx:base,q,strand\t%d\t%d\t%d\n",b,q,bam_is_rev(p->b)?0:1);
+
         // collect annotations
         if (b < 4)
         {
-            r->qsum[b] += q;
+            r->QS[b] += q;
             if ( r->ADF )
             {
                 if ( bam_is_rev(p->b) )
@@ -228,29 +409,75 @@ int bcf_call_glfgen(int _n, const bam_pileup1_t *pl, int ref_base, bcf_callaux_t
         // collect for bias tests
         if ( baseQ > 59 ) baseQ = 59;
         if ( mapQ > 59 ) mapQ = 59;
-        int len, epos = 0;
-        if ( bca->fmt_flag & (B2B_INFO_RPB|B2B_INFO_VDB) )
+        int len, epos = 0, sc_len = 0, sc_dist = 0;
+        if ( bca->fmt_flag & (B2B_INFO_RPBZ|B2B_INFO_VDB|B2B_INFO_SCBZ) )
         {
-            int pos = get_position(p, &len);
-            epos = (double)pos/(len+1) * bca->npos;
+            int pos = get_position(p, &len, &sc_len, &sc_dist);
+            epos = (double)pos/(len+1) * (bca->npos - 1);
+            if (sc_len) {
+                sc_len = 15.0*sc_len / (sc_dist+1);
+                if (sc_len > 99) sc_len = 99;
+            }
+            assert( epos>=0 && epos<bca->npos );
+            assert( sc_len>=0 && sc_len<bca->npos );
         }
-        int ibq  = baseQ/60. * bca->nqual;
-        int imq  = mapQ/60. * bca->nqual;
-        if ( bam_is_rev(p->b) ) bca->rev_mqs[imq]++;
-        else bca->fwd_mqs[imq]++;
-        if ( bam_seqi(bam_get_seq(p->b),p->qpos) == ref_base )
+        int imq  = mapQ * nqual_over_60;
+        int ibq  = baseQ * nqual_over_60;
+
+        if ( bam_is_rev(p->b) )
+            bca->rev_mqs[imq]++;
+        else
+            bca->fwd_mqs[imq]++;
+
+        if ( !is_diff )
         {
             bca->ref_pos[epos]++;
             bca->ref_bq[ibq]++;
             bca->ref_mq[imq]++;
+            bca->ref_scl[sc_len]++;
+            if ( inm>=0 )
+            {
+                bca->ref_nm[inm]++;
+                if ( r->ref_nm ) r->ref_nm[inm]++;
+            }
         }
         else
         {
             bca->alt_pos[epos]++;
             bca->alt_bq[ibq]++;
             bca->alt_mq[imq]++;
+            bca->alt_scl[sc_len]++;
+            if ( inm>=0 )
+            {
+                bca->alt_nm[inm]++;
+                if ( r->alt_nm ) r->alt_nm[inm]++;
+            }
         }
     }
+
+    // Compensate for AD not being counted on low quality REF indel matches.
+    if ( r->ADF && bca->ambig_reads==B2B_INC_AD0 )
+    {
+        for (i=0; i<4; i++)
+        {
+            r->ADR[0] += ADR_ref_missed[i];
+            r->ADF[0] += ADF_ref_missed[i];
+        }
+    }
+    else if ( r->ADF && bca->ambig_reads==B2B_INC_AD )
+    {
+        int dp = 0, dp_ambig = 0;
+        for (i=0; i<4; i++) dp += r->ADR[i];
+        for (i=0; i<4; i++) dp_ambig += ADR_ref_missed[i];
+        if ( dp )
+            for (i=0; i<4; i++) r->ADR[i] += lroundf((float)dp_ambig * r->ADR[i]/dp);
+        dp = 0, dp_ambig = 0;
+        for (i=0; i<4; i++) dp += r->ADF[i];
+        for (i=0; i<4; i++) dp_ambig += ADF_ref_missed[i];
+        if ( dp )
+            for (i=0; i<4; i++) r->ADF[i] += lroundf((float)dp_ambig * r->ADF[i]/dp);
+    }
+
     r->ori_depth = ori_depth;
     // glfgen
     errmod_cal(bca->e, n, 5, bca->bases, r->p); // calculate PL of each genotype
@@ -396,17 +623,18 @@ double mann_whitney_1947_cdf(int n, int m, int U)
 double calc_mwu_bias_cdf(int *a, int *b, int n)
 {
     int na = 0, nb = 0, i;
-    double U = 0, ties = 0;
+    double U = 0;
+    //double ties = 0;
     for (i=0; i<n; i++)
     {
         na += a[i];
         U  += a[i] * (nb + b[i]*0.5);
         nb += b[i];
-        if ( a[i] && b[i] )
-        {
-            double tie = a[i] + b[i];
-            ties += (tie*tie-1)*tie;
-        }
+        // if ( a[i] && b[i] )
+        // {
+        //     double tie = a[i] + b[i];
+        //     ties += (tie*tie-1)*tie;
+        // }
     }
     if ( !na || !nb ) return HUGE_VAL;
 
@@ -437,10 +665,11 @@ double calc_mwu_bias_cdf(int *a, int *b, int n)
     return pval>1 ? 1 : pval;
 }
 
-double calc_mwu_bias(int *a, int *b, int n)
+double calc_mwu_bias(int *a, int *b, int n, int left)
 {
     int na = 0, nb = 0, i;
-    double U = 0, ties = 0;
+    double U = 0;
+    // double ties = 0;
     for (i=0; i<n; i++)
     {
         if (!a[i]) {
@@ -453,14 +682,15 @@ double calc_mwu_bias(int *a, int *b, int n)
             na += a[i];
             U  += a[i] * (nb + b[i]*0.5);
             nb += b[i];
-            double tie = a[i] + b[i];
-            ties += (tie*tie-1)*tie;
+            // double tie = a[i] + b[i];
+            // ties += (tie*tie-1)*tie;
         }
     }
     if ( !na || !nb ) return HUGE_VAL;
     if ( na==1 || nb==1 ) return 1.0;       // Flat probability, all U values are equally likely
 
     double mean = ((double)na*nb)*0.5;
+    if (left && U > mean) return 1; // for MQB which is asymmetrical
     if ( na==2 || nb==2 )
     {
         // Linear approximation
@@ -483,6 +713,85 @@ double calc_mwu_bias(int *a, int *b, int n)
     return mann_whitney_1947(na,nb,U) * sqrt(2*M_PI*var2);
 }
 
+// A Z-score version of the above function.
+//
+// See "Normal approximation and tie correction" at
+// https://en.wikipedia.org/wiki/Mann%E2%80%93Whitney_U_test
+//
+// The Z score is the number of standard deviations above or below the mean
+// with 0 being equality of the two distributions and +ve/-ve from there.
+//
+// This is a more robust score to filter on.
+double calc_mwu_biasZ(int *a, int *b, int n, int left_only, int do_Z) {
+    int i;
+    int64_t t;
+
+    // Optimisation
+    for (i = 0; i < n; i++)
+        if (b[i])
+            break;
+    int b_empty = (i == n);
+
+    // Count equal (e), less-than (l) and greater-than (g) permutations.
+    int e = 0, l = 0, na = 0, nb = 0;
+    if (b_empty) {
+        for (t = 0, i = n-1; i >= 0; i--) {
+            na += a[i];
+            t += (a[i]*a[i]-1)*a[i];  // adjustment score for ties
+        }
+    } else {
+        for (t = 0, i = n-1; i >= 0; i--) {
+            // Combinations of a[i] and b[j] for i==j
+            e += a[i]*b[i];
+
+            // nb is running total of b[i+1]..b[n-1].
+            // Therefore a[i]*nb is the number of combinations of a[i] and b[j]
+            // for all i < j.
+            l += a[i]*nb;    // a<b
+
+            na += a[i];
+            nb += b[i];
+            int p = a[i]+b[i];
+            t += (p*p-1)*p;  // adjustment score for ties
+        }
+    }
+
+    if (!na || !nb)
+        return HUGE_VAL;
+
+    double U, m;
+    U = l + e*0.5; // Mann-Whitney U score
+    m = na*nb / 2.0;
+
+    // With ties adjustment
+    double var2 = (na*nb)/12.0 * ((na+nb+1) - t/(double)((na+nb)*(na+nb-1)));
+    // var = na*nb*(na+nb+1)/12.0; // simpler; minus tie adjustment
+    if (var2 <= 0)
+        return do_Z ? 0 : 1;
+
+    if (do_Z) {
+        // S.D. normalised Z-score
+        //Z = (U - m - (U-m >= 0 ? 0.5 : -0.5)) / sd; // gatk method?
+        return (U - m) / sqrt(var2);
+    }
+
+    // Else U score, which can be asymmetric for some data types.
+    if (left_only && U > m)
+        return HUGE_VAL; // one-sided, +ve bias is OK, -ve is not.
+
+    if (na >= 8 || nb >= 8) {
+        // Normal approximation, very good for na>=8 && nb>=8 and
+        // reasonable if na<8 or nb<8
+        return exp(-0.5*(U-m)*(U-m)/var2);
+    }
+
+    // Exact calculation
+    if (na==1 || nb == 1)
+        return mann_whitney_1947_(na, nb, U) * sqrt(2*M_PI*var2);
+    else
+        return mann_whitney_1947(na, nb, U) * sqrt(2*M_PI*var2);
+}
+
 static inline double logsumexp2(double a, double b)
 {
     if ( a>b )
@@ -558,7 +867,7 @@ void calc_SegBias(const bcf_callret1_t *bcr, bcf_call_t *call)
 int bcf_call_combine(int n, const bcf_callret1_t *calls, bcf_callaux_t *bca, int ref_base /*4-bit*/, bcf_call_t *call)
 {
     int ref4, i, j;
-    float qsum[5] = {0,0,0,0,0};
+    float qsum[B2B_MAX_ALLELES] = {0,0,0,0,0};
     if (ref_base >= 0) {
         call->ori_ref = ref4 = seq_nt16_int[ref_base];
         if (ref4 > 4) ref4 = 4;
@@ -569,9 +878,9 @@ int bcf_call_combine(int n, const bcf_callret1_t *calls, bcf_callaux_t *bca, int
     for (i = 0; i < n; ++i)
     {
         float sum = 0;
-        for (j = 0; j < 4; ++j) sum += calls[i].qsum[j];
+        for (j = 0; j < 4; ++j) sum += calls[i].QS[j];
         if ( sum )
-            for (j = 0; j < 4; j++) qsum[j] += calls[i].qsum[j] / sum;
+            for (j = 0; j < 4; j++) qsum[j] += (float)calls[i].QS[j] / sum;
     }
 
     // sort qsum in ascending order (insertion sort)
@@ -583,7 +892,7 @@ int bcf_call_combine(int n, const bcf_callret1_t *calls, bcf_callaux_t *bca, int
 
     // Set the reference allele and alternative allele(s)
     for (i=0; i<5; i++) call->a[i] = -1;
-    for (i=0; i<5; i++) call->qsum[i] = 0;
+    for (i=0; i<B2B_MAX_ALLELES; i++) call->qsum[i] = 0;
     call->unseen = -1;
     call->a[0] = ref4;
     for (i=3, j=1; i>=0; i--)   // i: alleles sorted by QS; j, a[j]: output allele ordering
@@ -610,6 +919,7 @@ int bcf_call_combine(int n, const bcf_callret1_t *calls, bcf_callaux_t *bca, int
         call->n_alleles = j;
         if (call->n_alleles == 1) return -1; // no reliable supporting read. stop doing anything
     }
+    int has_alt = (call->n_alleles==2 && call->unseen!=-1) ? 0 : 1;
     /*
      * Set the phread likelihood array (call->PL) This array is 15 entries long
      * for each sample because that is size of an upper or lower triangle of a
@@ -695,8 +1005,24 @@ int bcf_call_combine(int n, const bcf_callret1_t *calls, bcf_callaux_t *bca, int
                 adf += B2B_MAX_ALLELES;
             }
         }
+        if ( bca->fmt_flag & B2B_FMT_QS )
+        {
+            assert( call->n_alleles<=B2B_MAX_ALLELES );   // this is always true for SNPs and so far for indels as well
+
+            // reorder QS to match the allele ordering at this site
+            int32_t tmp[B2B_MAX_ALLELES];
+            int32_t *qs = call->QS, *qs_out = call->QS;
+            for (i=0; i<n; i++)
+            {
+                for (j=0; j<call->n_alleles; j++) tmp[j] = qs[ call->a[j] ];
+                for (j=0; j<call->n_alleles; j++) qs_out[j] = tmp[j] < BCF_MAX_BT_INT32 ? tmp[j] : BCF_MAX_BT_INT32;
+                qs_out += call->n_alleles;
+                qs += B2B_MAX_ALLELES;
+            }
+        }
 
 //      if (ref_base < 0) fprintf(stderr, "%d,%d,%f,%d\n", call->n_alleles, x, sum_min, call->unseen);
+        // fprintf(stderr,"sum_min=%f\n",sum_min);
         call->shift = (int)(sum_min + .499);
     }
     // combine annotations
@@ -711,28 +1037,51 @@ int bcf_call_combine(int n, const bcf_callret1_t *calls, bcf_callaux_t *bca, int
         for (j = 0; j < 16; ++j) call->anno[j] += calls[i].anno[j];
     }
 
-    calc_SegBias(calls, call);
+    // No need to calculate MWU tests when there is no ALT allele, this should speed up things slightly
+    if ( !has_alt ) return 0;
+
+    if ( bca->fmt_flag & B2B_INFO_FS )
+    {
+        double left,right,two;
+        call->strand_bias = kt_fisher_exact(call->anno[0], call->anno[1], call->anno[2], call->anno[3], &left, &right, &two);
+    }
+    if ( bca->fmt_flag & B2B_INFO_SGB ) calc_SegBias(calls, call);
 
     // calc_chisq_bias("XPOS", call->bcf_hdr->id[BCF_DT_CTG][call->tid].key, call->pos, bca->ref_pos, bca->alt_pos, bca->npos);
     // calc_chisq_bias("XMQ", call->bcf_hdr->id[BCF_DT_CTG][call->tid].key, call->pos, bca->ref_mq, bca->alt_mq, bca->nqual);
     // calc_chisq_bias("XBQ", call->bcf_hdr->id[BCF_DT_CTG][call->tid].key, call->pos, bca->ref_bq, bca->alt_bq, bca->nqual);
 
-    if ( bca->fmt_flag & B2B_INFO_RPB )
-        call->mwu_pos = calc_mwu_bias(bca->ref_pos, bca->alt_pos, bca->npos);
-    call->mwu_mq  = calc_mwu_bias(bca->ref_mq,  bca->alt_mq,  bca->nqual);
-    call->mwu_bq  = calc_mwu_bias(bca->ref_bq,  bca->alt_bq,  bca->nqual);
-    call->mwu_mqs = calc_mwu_bias(bca->fwd_mqs, bca->rev_mqs, bca->nqual);
-
-#if CDF_MWU_TESTS
-    // CDF version of MWU tests is not calculated by default
-    if ( bca->fmt_flag & B2B_INFO_RPB )
-        call->mwu_pos_cdf = calc_mwu_bias_cdf(bca->ref_pos, bca->alt_pos, bca->npos);
-    call->mwu_mq_cdf  = calc_mwu_bias_cdf(bca->ref_mq,  bca->alt_mq,  bca->nqual);
-    call->mwu_bq_cdf  = calc_mwu_bias_cdf(bca->ref_bq,  bca->alt_bq,  bca->nqual);
-    call->mwu_mqs_cdf = calc_mwu_bias_cdf(bca->fwd_mqs, bca->rev_mqs, bca->nqual);
-#endif
-
-    if ( bca->fmt_flag & B2B_INFO_VDB ) 
+    // U z-normalised as +/- number of standard deviations from mean.
+    if (call->ori_ref < 0) {    // indel
+        if ( bca->fmt_flag & B2B_INFO_RPBZ )
+            call->mwu_pos = calc_mwu_biasZ(bca->iref_pos, bca->ialt_pos, bca->npos, 0, 1);
+        if ( bca->fmt_flag & B2B_INFO_MQBZ )
+            call->mwu_mq  = calc_mwu_biasZ(bca->iref_mq,  bca->ialt_mq, bca->nqual,1,1);
+        if ( bca->fmt_flag & B2B_INFO_SCBZ )
+            call->mwu_sc  = calc_mwu_biasZ(bca->iref_scl, bca->ialt_scl, 100, 0,1);
+    } else {
+        if ( bca->fmt_flag & B2B_INFO_RPBZ )
+            call->mwu_pos = calc_mwu_biasZ(bca->ref_pos, bca->alt_pos, bca->npos, 0, 1);
+        if ( bca->fmt_flag & B2B_INFO_MQBZ )
+            call->mwu_mq  = calc_mwu_biasZ(bca->ref_mq,  bca->alt_mq, bca->nqual,1,1);
+        if ( bca->fmt_flag & B2B_INFO_BQBZ )
+            call->mwu_bq  = calc_mwu_biasZ(bca->ref_bq,  bca->alt_bq, bca->nqual,0,1);
+        if ( bca->fmt_flag & B2B_INFO_MQSBZ )
+            call->mwu_mqs = calc_mwu_biasZ(bca->fwd_mqs, bca->rev_mqs, bca->nqual,0,1);
+        if ( bca->fmt_flag & B2B_INFO_SCBZ )
+            call->mwu_sc  = calc_mwu_biasZ(bca->ref_scl, bca->alt_scl, 100, 0,1);
+    }
+    if ( bca->fmt_flag & B2B_INFO_NMBZ )
+        call->mwu_nm[0] = calc_mwu_biasZ(bca->ref_nm, bca->alt_nm, B2B_N_NM,0,1);
+    if ( bca->fmt_flag & B2B_FMT_NMBZ )
+    {
+        for (i=0; i<n; i++)
+        {
+            float val = calc_mwu_biasZ(calls[i].ref_nm, calls[i].alt_nm, B2B_N_NM,0,1);
+            call->mwu_nm[i+1] = val!=HUGE_VAL ? val : 0;
+        }
+    }
+    if ( bca->fmt_flag & B2B_INFO_VDB )
         call->vdb = calc_vdb(bca->alt_pos, bca->npos);
 
     return 0;
@@ -741,7 +1090,7 @@ int bcf_call_combine(int n, const bcf_callret1_t *calls, bcf_callaux_t *bca, int
 int bcf_call2bcf(bcf_call_t *bc, bcf1_t *rec, bcf_callret1_t *bcr, int fmt_flag, const bcf_callaux_t *bca, const char *ref)
 {
     extern double kt_fisher_exact(int n11, int n12, int n21, int n22, double *_left, double *_right, double *two);
-    int i, j, nals = 1;
+    int i, j, nals = 1, has_alt = 0;
 
     bcf_hdr_t *hdr = bc->bcf_hdr;
     rec->rid  = bc->tid;
@@ -771,6 +1120,7 @@ int bcf_call2bcf(bcf_call_t *bc, bcf1_t *rec, bcf_callret1_t *bcr, int fmt_flag,
                 for (j = 0; j < bca->indelreg; ++j) kputc(ref[bc->pos+1+j], &bc->tmp);
             }
             nals++;
+            has_alt = 1;
         }
     }
     else    // SNP
@@ -781,7 +1131,11 @@ int bcf_call2bcf(bcf_call_t *bc, bcf1_t *rec, bcf_callret1_t *bcr, int fmt_flag,
             if (bc->a[i] < 0) break;
             kputc(',', &bc->tmp);
             if ( bc->unseen==i ) kputs("<*>", &bc->tmp);
-            else kputc("ACGT"[bc->a[i]], &bc->tmp);
+            else
+            {
+                kputc("ACGT"[bc->a[i]], &bc->tmp);
+                has_alt = 1;
+            }
             nals++;
         }
     }
@@ -790,11 +1144,13 @@ int bcf_call2bcf(bcf_call_t *bc, bcf1_t *rec, bcf_callret1_t *bcr, int fmt_flag,
     bc->tmp.l = 0;
 
     // INFO
-    if (bc->ori_ref < 0)
+    if ( bc->ori_ref < 0 )
     {
         bcf_update_info_flag(hdr, rec, "INDEL", NULL, 1);
-        bcf_update_info_int32(hdr, rec, "IDV", &bca->max_support, 1);
-        bcf_update_info_float(hdr, rec, "IMF", &bca->max_frac, 1);
+        if ( fmt_flag&B2B_INFO_IDV )
+            bcf_update_info_int32(hdr, rec, "IDV", &bca->max_support, 1);
+        if ( fmt_flag&B2B_INFO_IMF )
+            bcf_update_info_float(hdr, rec, "IMF", &bca->max_frac, 1);
     }
     bcf_update_info_int32(hdr, rec, "DP", &bc->ori_depth, 1);
     if ( fmt_flag&B2B_INFO_ADF )
@@ -817,20 +1173,39 @@ int bcf_call2bcf(bcf_call_t *bc, bcf1_t *rec, bcf_callret1_t *bcr, int fmt_flag,
     bcf_update_info_float(hdr, rec, "I16", tmpf, 16);
     bcf_update_info_float(hdr, rec, "QS", bc->qsum, nals);
 
-    if ( bc->vdb != HUGE_VAL )      bcf_update_info_float(hdr, rec, "VDB", &bc->vdb, 1);
-    if ( bc->seg_bias != HUGE_VAL ) bcf_update_info_float(hdr, rec, "SGB", &bc->seg_bias, 1);
-    if ( bc->mwu_pos != HUGE_VAL )  bcf_update_info_float(hdr, rec, "RPB", &bc->mwu_pos, 1);
-    if ( bc->mwu_mq != HUGE_VAL )   bcf_update_info_float(hdr, rec, "MQB", &bc->mwu_mq, 1);
-    if ( bc->mwu_mqs != HUGE_VAL )  bcf_update_info_float(hdr, rec, "MQSB", &bc->mwu_mqs, 1);
-    if ( bc->mwu_bq != HUGE_VAL )   bcf_update_info_float(hdr, rec, "BQB", &bc->mwu_bq, 1);
-#if CDF_MWU_TESTS
-    if ( bc->mwu_pos_cdf != HUGE_VAL )  bcf_update_info_float(hdr, rec, "RPB2", &bc->mwu_pos_cdf, 1);
-    if ( bc->mwu_mq_cdf != HUGE_VAL )   bcf_update_info_float(hdr, rec, "MQB2", &bc->mwu_mq_cdf, 1);
-    if ( bc->mwu_mqs_cdf != HUGE_VAL )  bcf_update_info_float(hdr, rec, "MQSB2", &bc->mwu_mqs_cdf, 1);
-    if ( bc->mwu_bq_cdf != HUGE_VAL )   bcf_update_info_float(hdr, rec, "BQB2", &bc->mwu_bq_cdf, 1);
-#endif
+    if ( has_alt )
+    {
+        if ( fmt_flag&B2B_INFO_MIN_PL_SUM )
+            bcf_update_info_int32(hdr, rec, "MIN_PL_SUM", &bc->shift, 1);
+        if ( fmt_flag&B2B_INFO_VDB && bc->vdb != HUGE_VAL )
+            bcf_update_info_float(hdr, rec, "VDB", &bc->vdb, 1);
+        if ( fmt_flag&B2B_INFO_SGB && bc->seg_bias != HUGE_VAL )
+            bcf_update_info_float(hdr, rec, "SGB", &bc->seg_bias, 1);
+        if ( fmt_flag&B2B_INFO_NM && (bca->nnm[0] || bca->nnm[1]) )
+        {
+            for (i=0; i<2; i++) bc->nm[i] = bca->nnm[i] ? bca->nm[i]/bca->nnm[i] : 0;
+            bcf_update_info_float(hdr, rec, "NM", bc->nm, 2);
+        }
+
+        if ( fmt_flag&B2B_INFO_RPBZ && bc->mwu_pos != HUGE_VAL )
+            bcf_update_info_float(hdr, rec, "RPBZ", &bc->mwu_pos, 1);
+        if ( fmt_flag&B2B_INFO_MQBZ && bc->mwu_mq != HUGE_VAL )
+            bcf_update_info_float(hdr, rec, "MQBZ", &bc->mwu_mq, 1);
+        if ( fmt_flag&B2B_INFO_MQSBZ && bc->mwu_mqs != HUGE_VAL )
+            bcf_update_info_float(hdr, rec, "MQSBZ", &bc->mwu_mqs, 1);
+        if ( fmt_flag&B2B_INFO_BQBZ && bc->mwu_bq != HUGE_VAL )
+            bcf_update_info_float(hdr, rec, "BQBZ", &bc->mwu_bq, 1);
+        if ( fmt_flag&B2B_INFO_NMBZ && bc->mwu_nm[0] != HUGE_VAL )
+            bcf_update_info_float(hdr, rec, "NMBZ", bc->mwu_nm, 1);
+        if ( fmt_flag&B2B_INFO_SCBZ && bc->mwu_sc != HUGE_VAL )
+            bcf_update_info_float(hdr, rec, "SCBZ", &bc->mwu_sc, 1);
+        if ( fmt_flag&B2B_INFO_FS && bc->strand_bias != HUGE_VAL )
+            bcf_update_info_float(hdr, rec, "FS", &bc->strand_bias, 1);
+    }
+
     tmpf[0] = bc->ori_depth ? (float)bc->mq0/bc->ori_depth : 0;
-    bcf_update_info_float(hdr, rec, "MQ0F", tmpf, 1);
+    if ( fmt_flag&B2B_INFO_MQ0F )
+        bcf_update_info_float(hdr, rec, "MQ0F", tmpf, 1);
 
     // FORMAT
     rec->n_sample = bc->n;
@@ -884,6 +1259,14 @@ int bcf_call2bcf(bcf_call_t *bc, bcf1_t *rec, bcf_callret1_t *bcr, int fmt_flag,
     }
     if ( fmt_flag&B2B_FMT_SCR )
         bcf_update_format_int32(hdr, rec, "SCR", bc->SCR+1, rec->n_sample);
+    if ( fmt_flag&B2B_FMT_QS )
+        bcf_update_format_int32(hdr, rec, "QS", bc->QS, rec->n_sample*rec->n_allele);
+
+    if ( has_alt )
+    {
+        if ( fmt_flag&B2B_FMT_NMBZ )
+            bcf_update_format_float(hdr, rec, "NMBZ", bc->mwu_nm+1, rec->n_sample);
+    }
 
     return 0;
 }
