@@ -1,6 +1,6 @@
 /*  convert.c -- functions for converting between VCF/BCF and related formats.
 
-    Copyright (C) 2013-2022 Genome Research Ltd.
+    Copyright (C) 2013-2023 Genome Research Ltd.
 
     Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -104,8 +104,11 @@ struct _convert_t
     char *undef_info_tag;
     void *used_tags_hash;
     char **used_tags_list;
+    char *print_filtered;
     int nused_tags;
     int allow_undef_tags;
+    int force_newline;
+    int header_samples;
     uint8_t **subset_samples;
 };
 
@@ -648,6 +651,7 @@ static void process_type(convert_t *convert, bcf1_t *line, fmt_t *fmt, int isamp
 static void process_line(convert_t *convert, bcf1_t *line, fmt_t *fmt, int isample, kstring_t *str)
 {
     vcf_format1(convert->header, line, str);
+    if ( str->s[str->l-1]=='\n' ) str->l--;
 }
 static void process_chrom_pos_id(convert_t *convert, bcf1_t *line, fmt_t *fmt, int isample, kstring_t *str)
 {
@@ -1117,14 +1121,21 @@ static void process_rsid_hex(convert_t *convert, bcf1_t *line, fmt_t *fmt, int i
 
 static void process_variantkey_hex(convert_t *convert, bcf1_t *line, fmt_t *fmt, int isample, kstring_t *str)
 {
+    const char *alt = NULL;
+    size_t sizealt = 0;
+    if ( line->n_allele>1 )
+    {
+        alt = line->d.allele[1];
+        sizealt = strlen(line->d.allele[1]);
+    }
     uint64_t vk = variantkey(
         convert->header->id[BCF_DT_CTG][line->rid].key,
         strlen(convert->header->id[BCF_DT_CTG][line->rid].key),
         line->pos,
         line->d.allele[0],
         strlen(line->d.allele[0]),
-        line->d.allele[1],
-        strlen(line->d.allele[1]));
+        alt,
+        sizealt);
     ksprintf(str, "%016" PRIx64 "", vk);
 }
 
@@ -1541,6 +1552,7 @@ void convert_destroy(convert_t *convert)
         free(convert->used_tags_list);
     }
     khash_str2int_destroy(convert->used_tags_hash);
+    free(convert->print_filtered);
     free(convert->fmt);
     free(convert->undef_info_tag);
     free(convert->dat);
@@ -1561,7 +1573,13 @@ int convert_header(convert_t *convert, kstring_t *str)
     if ( i!=convert->nfmt )
         return str->l - l_ori;
 
-    kputs("# ", str);
+    // Header formatting becomes problematic when the formatting expression contains a newline.
+    // Simple cases like
+    //      -f'[%CHROM %POS %SAMPLE\n]'
+    // can be handled quite easily with has_fmt_newline. Note this will not work if multiple newlines
+    // are present.
+    int has_fmt_newline = 0;
+    kputc('#', str);
     for (i=0; i<convert->nfmt; i++)
     {
         // Genotype fields
@@ -1576,12 +1594,31 @@ int convert_header(convert_t *convert, kstring_t *str)
                 {
                     if ( convert->fmt[k].type == T_SEP )
                     {
-                        if ( convert->fmt[k].key ) kputs(convert->fmt[k].key, str);
+                        if ( convert->fmt[k].key )
+                        {
+                            char *tmp = convert->fmt[k].key;
+                            while ( *tmp )
+                            {
+                                if ( *tmp=='\n' ) has_fmt_newline = 1;
+                                else kputc(*tmp,str);
+                                tmp++;
+                            }
+                        }
                     }
-                    else if ( convert->fmt[k].type == T_SAMPLE )
-                        ksprintf(str, "[%d]%s", ++icol, convert->fmt[k].key);
-                    else
+                    else if ( convert->header_samples )
                         ksprintf(str, "[%d]%s:%s", ++icol, hdr->samples[ks], convert->fmt[k].key);
+                    else
+                        ksprintf(str, "[%d]%s", ++icol, convert->fmt[k].key);
+                }
+                if ( has_fmt_newline )
+                {
+                    if ( !convert->header_samples ) break;
+
+                    // this is unfortunate: the formatting expression breaks the per-sample output into separate lines,
+                    // therefore including a sample name in the header makes no sense anymore
+                    convert->header_samples = 0;
+                    str->l = l_ori;
+                    return convert_header(convert, str);
                 }
             }
             i = j-1;
@@ -1595,6 +1632,7 @@ int convert_header(convert_t *convert, kstring_t *str)
         }
         ksprintf(str, "[%d]%s", ++icol, convert->fmt[i].key);
     }
+    if ( has_fmt_newline ) kputc('\n',str);
     return str->l - l_ori;
 }
 
@@ -1631,7 +1669,17 @@ int convert_line(convert_t *convert, bcf1_t *line, kstring_t *str)
             {
                 // Skip samples when filtering was requested
                 int ks = convert->samples[js];
-                if ( convert->subset_samples && *convert->subset_samples && !(*convert->subset_samples)[ks] ) continue;
+                if ( convert->subset_samples && *convert->subset_samples && !(*convert->subset_samples)[ks] )
+                {
+                    if ( !convert->print_filtered ) continue;
+
+                    for (k=i; k<j; k++)
+                        if ( convert->fmt[k].type==T_SEP )
+                            convert->fmt[k].handler(convert, line, &convert->fmt[k], ks, str);
+                        else
+                            kputs(convert->print_filtered, str);
+                    continue;
+                }
 
                 // Here comes a hack designed for TBCSQ. When running on large files,
                 // such as 1000GP, there are too many empty fields in the output and
@@ -1671,6 +1719,36 @@ int convert_line(convert_t *convert, bcf1_t *line, kstring_t *str)
     return str->l - l_ori;
 }
 
+static void force_newline_(convert_t *convert)
+{
+    int i, has_newline = 0;
+    for (i=0; i<convert->nfmt; i++)
+    {
+        if ( !convert->fmt[i].key ) continue;
+        char *tmp = convert->fmt[i].key;
+        while (*tmp)
+        {
+            if ( *tmp=='\n' ) { has_newline = 1; break; }
+            tmp++;
+        }
+        if ( has_newline ) break;
+    }
+    if ( has_newline ) return;
+
+    // A newline is not present, force it. But where to add it? Always at the end.
+    //
+    // Briefly, in 1.18, we considered the following automatic behavior, which for
+    // per-site output it would add it at the end of the expression and for per-sample
+    // output it would add it inside the square brackets:
+    //           -f'%CHROM[ %SAMPLE]\n'
+    //           -f'[%CHROM %SAMPLE\n]'
+    //
+    // However, this is an annoyance for users, as it is not entirely clear what
+    // will happen unless one understands the internals well (#1969)
+
+    register_tag(convert, "\n", 0, T_SEP);
+}
+
 int convert_set_option(convert_t *convert, enum convert_option opt, ...)
 {
     int ret = 0;
@@ -1685,6 +1763,16 @@ int convert_set_option(convert_t *convert, enum convert_option opt, ...)
         case subset_samples:
             convert->subset_samples = va_arg(args, uint8_t**);
             break;
+        case header_samples:
+            convert->header_samples = va_arg(args, int);
+            break;
+        case print_filtered:
+            convert->print_filtered = strdup(va_arg(args, char*));
+            break;
+        case force_newline:
+            convert->force_newline = va_arg(args, int);
+            if ( convert->force_newline ) force_newline_(convert);
+            break;
         default:
             ret = -1;
     }
