@@ -1,19 +1,19 @@
 /* The MIT License
 
-   Copyright (c) 2018-2021 Genome Research Ltd.
+   Copyright (c) 2018-2023 Genome Research Ltd.
 
    Author: Petr Danecek <pd3@sanger.ac.uk>
-   
+
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:
-   
+
    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.
-   
+
    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -82,6 +82,13 @@ typedef struct
 }
 priors_t;
 
+typedef struct
+{
+    double abs, frac;
+    double abs1, frac1;   // applied only if allele observed in a single parent, but not when observed in both
+}
+pnoise_t;
+
 typedef struct
 {
     int argc, filter_logic, regions_is_file, targets_is_file, output_type, record_cmd_line, clevel;
@@ -112,17 +119,17 @@ typedef struct
          *dnm_allele_tag;
     int dnm_score_type;             // given by e.g. --dnm-tag DNM:log
     double mrate;                   // --mrate, mutation rate
-    double pn_abs,pn_frac;          // --pn
-    double pns_abs,pns_frac;        // --pns
+    pnoise_t pn_snv, pn_indel;      // --pn and --pns for SNVs and indels
     int with_ppl, with_pad;         // --with-pPL or --with-pAD
     int use_dng_priors;             // --dng-priors
     int need_QS;
+    int strictly_novel;
     priors_t priors, priors_X, priors_XX;
+    char *index_fn;
+    int write_index;
 }
 args_t;
 
-args_t args;
-
 const char *about(void)
 {
     return "Screen variants for possible de-novo mutations in trios.\n";
@@ -130,7 +137,7 @@ const char *about(void)
 
 static const char *usage_text(void)
 {
-    return 
+    return
         "\n"
         "About: Screen variants for possible de-novo mutations in trios\n"
         "Usage: bcftools +trio-dnm2 [OPTIONS]\n"
@@ -166,12 +173,15 @@ static const char *usage_text(void)
         "Model options:\n"
         "       --dng-priors                Use the original DeNovoGear priors (including bugs in prior assignment, but with chrX bugs fixed)\n"
         "       --mrate NUM                 Mutation rate [1e-8]\n"
-        "       --pn FRAC[,NUM]             Tolerance to parental noise or mosaicity, given as fraction of QS or number of reads [0.005,0]\n"
-        "       --pns FRAC[,NUM]            Same as --pn but is not applied to alleles observed in both parents (fewer FPs, more FNs) [0.045,0]\n"
+        "       --pn FRAC[,NUM][:type]      Tolerance to parental noise or mosaicity, given as fraction of QS or number of reads. The type is\n"
+        "                                       'snv', 'indel' or 'both' [--pn 0.005,0:snv --pn 0,0:indel]\n"
+        "       --pns FRAC[,NUM][:type]     Same as --pn but applied for alleles observed in a single parent [--pns 0.045,0:snv --pns 0,0:indel]\n"
+        "   -n, --strictly-novel            When Mendelian inheritance is violiated, score highly only novel alleles (e.g. in LoH regions)\n"
         "       --use-DNG                   The original DeNovoGear model, implies --dng-priors\n"
         "       --use-NAIVE                 A naive calling model which uses only FMT/GT to determine DNMs\n"
         "       --with-pAD                  Do not use FMT/QS but parental FMT/AD\n"
         "       --with-pPL                  Do not use FMT/QS but parental FMT/PL. Equals to DNG with bugs fixed (more FPs, fewer FNs)\n"
+        "       --write-index               Automatically index the output files [off]\n"
         "\n"
         "Example:\n"
         "   # Annotate VCF with FORMAT/DNM, run for a single trio\n"
@@ -365,7 +375,7 @@ static double init_mf_priors(args_t *args, int fi, int mi)
         gt_prior = p_poly / 57.;
     else if ( nref_mf==4 )                                          // 0 ALTs; 00,00
         gt_prior = p_homref;
-    else if ( nref_mf==3 )                                          // this and all remaining have 1 unique ALT allele; 00,0x 
+    else if ( nref_mf==3 )                                          // this and all remaining have 1 unique ALT allele; 00,0x
         gt_prior = p_nonref * (4.0/15.0) * (1.0/3.0);
     else if ( nref_mf==2 && ma==mb )                                // hom alt; 00,xx
         gt_prior = p_nonref * (2.0/15.0) * (1.0/3.0);
@@ -466,7 +476,7 @@ static void init_DNG_tprob_mprob(args_t *args, int fi, int mi, int ci, double *t
         }
         else
         {
-            if ( ca!=fa  && ca!=fb && ca!=ma  && ca!=mb && 
+            if ( ca!=fa  && ca!=fb && ca!=ma  && ca!=mb &&
                     cb!=fa  && cb!=fb && cb!=ma  && cb!=mb ) *mprob = args->mrate * args->mrate;    // two mutations
             else
                 *mprob = args->mrate;
@@ -502,7 +512,17 @@ static void init_tprob_mprob(args_t *args, int fi, int mi, int ci, double *tprob
     // tprob .. genotype transmission probability L(GC|GM,GF), 0 if not compatible with Mendelian inheritance
     // mprob .. probability of mutation
 
-    if ( ((ca==fa||ca==fb) && (cb==ma||cb==mb)) || ((ca==ma||ca==mb) && (cb==fa||cb==fb)) )
+    int is_novel;
+    if ( args->strictly_novel )     // account for LoH sites, see chr1:10000057 in trio-dnm.11.vcf
+    {
+        is_novel = ( (ca!=fa && ca!=fb && ca!=ma && ca!=mb) || (cb!=fa && cb!=fb && cb!=ma && cb!=mb) ) ? 1 : 0;
+    }
+    else
+    {
+        is_novel = ( ((ca==fa||ca==fb) && (cb==ma||cb==mb)) || ((ca==ma||ca==mb) && (cb==fa||cb==fb)) ) ? 0 : 1;
+    }
+
+    if ( !is_novel )
     {
         if ( fa==fb && ma==mb ) *tprob = 1;
         else if ( fa==fb || ma==mb ) *tprob = 0.5;
@@ -547,8 +567,12 @@ static void init_tprob_mprob_chrXX(args_t *args, int fi, int mi, int ci, double
 
     *denovo_allele = ca!=fa && ca!=fb && ca!=ma && ca!=mb ? ca : cb;
 
-    if ( fa!=fb )                   // father cannot be heterozygous in X
-        *mprob = 0, *tprob = 0;
+    if ( fa!=fb )
+    {
+        // this must be a genotype error, father cannot be heterozygous in X; don't flag it as a DNM unless
+        // also autosomal inheritance fails
+        init_tprob_mprob(args,fi,mi,ci,tprob,mprob,denovo_allele);
+    }
     else if ( (ca==fa && (cb==ma||cb==mb)) || (cb==fa && (ca==ma||ca==mb)) )
     {
         if ( ma==mb ) *tprob = 1;
@@ -590,7 +614,7 @@ static void init_priors(args_t *args, priors_t *priors, init_priors_t type)
 
                 if ( args->use_dng_priors )
                     init_DNG_tprob_mprob(args,fi,mi,ci,&tprob,&mprob,&allele);
-                else if ( type==autosomal )
+                else if ( type==autosomal || args->strictly_novel )
                     init_tprob_mprob(args,fi,mi,ci,&tprob,&mprob,&allele);
                 else if ( type==chrX )
                     init_tprob_mprob_chrX(args,mi,ci,&tprob,&mprob,&allele);
@@ -750,6 +774,7 @@ static void init_data(args_t *args)
     args->out_fh = hts_open(args->output_fname ? args->output_fname : "-", wmode);
     if ( args->out_fh == NULL ) error("Can't write to \"%s\": %s\n", args->output_fname, strerror(errno));
     if ( bcf_hdr_write(args->out_fh, args->hdr_out)!=0 ) error("[%s] Error: cannot write to %s\n", __func__,args->output_fname);
+    if ( args->write_index && init_index(args->out_fh,args->hdr_out,args->output_fname,&args->index_fn)<0 ) error("Error: failed to initialise index for %s\n",args->output_fname);
 
     if ( args->dnm_score_type & DNM_FLOAT )
         args->dnm_qual_float = (float*) malloc(sizeof(*args->dnm_qual_float)*bcf_hdr_nsamples(args->hdr));
@@ -779,6 +804,15 @@ static void destroy_data(args_t *args)
     free(args->ad);
     free(args->qs);
     free(args->qs3);
+    if ( args->write_index )
+    {
+        if ( bcf_idx_save(args->out_fh)<0 )
+        {
+            if ( hts_close(args->out_fh)!=0 ) error("Error: close failed .. %s\n", args->output_fname?args->output_fname:"stdout");
+            error("Error: cannot write to index %s\n", args->index_fn);
+        }
+        free(args->index_fn);
+    }
     if ( hts_close(args->out_fh)!=0 ) error("[%s] Error: close failed .. %s\n", __func__,args->output_fname);
     bcf_hdr_destroy(args->hdr_out);
     bcf_sr_destroy(args->sr);
@@ -849,7 +883,7 @@ static double process_trio_ACM(args_t *args, priors_t *priors, int nals, double
                             else if ( fa==fb )
                                 fpl += qs[iFATHER][i];
                         }
-                    } 
+                    }
                     int mi = 0;
                     for (ma=0; ma<nals; ma++)
                     {
@@ -875,7 +909,7 @@ static double process_trio_ACM(args_t *args, priors_t *priors, int nals, double
                             sum = sum_log(sum,val);
 #define DEBUG 0
 #if DEBUG
-                            if(val!=-HUGE_VAL)                            
+                            if(val!=-HUGE_VAL)
                                 fprintf(stderr,"m,f,c: %d%d+%d%d=%d%d  dn=%d (%d,%d,%d)   mpl,fpl,cpl: %+e %+e %+e \t prior:%+e \t pval=%+e  sum=%+e  %c\n",
                                     mb,ma,fb,fa,cb,ca,priors->denovo[fi][mi][ci],fi,mi,ci,mpl,fpl,cpl,priors->pprob[fi][mi][ci], val,sum,(priors->denovo[fi][mi][ci] && max < val)?'*':'-');
 #endif
@@ -927,7 +961,7 @@ static double process_trio_DNG(args_t *args, priors_t *priors, int nals, double
                             val = pl[iCHILD][ci] + pl[iFATHER][fi] + pl[iMOTHER][mi] + priors->pprob[fi][mi][ci];
                             sum = sum_log(val,sum);
 #if DEBUG
-                            if(val!=-HUGE_VAL)                            
+                            if(val!=-HUGE_VAL)
                                 fprintf(stderr,"m,f,c: %d%d+%d%d=%d%d  dn=%d (%d,%d,%d)   mpl,fpl,cpl: %+e %+e %+e \t prior:%+e \t pval=%+e  sum=%+e  %c\n",
                                     mb,ma,fb,fa,cb,ca,priors->denovo[fi][mi][ci],fi,mi,ci,pl[iMOTHER][mi],pl[iFATHER][fi],pl[iCHILD][ci],priors->pprob[fi][mi][ci], val,sum,(priors->denovo[fi][mi][ci] && max < val)?'*':'-');
 #endif
@@ -1029,7 +1063,7 @@ static void many_alts_trim(args_t *args, int *_nals, double *pl[3], int *_npl, d
     for (i=2; i<nals; i++)
         for (j=i; j>1 && arr[idx[j]] < arr[idx[j-1]]; j--)
             tmp = idx[j], idx[j] = idx[j-1], idx[j-1] = tmp;
-    
+
     for (i=0; i<3; i++)
     {
         for (j=0; j<4; j++) args->alt_tmp[j] = qs[i][args->alt_idx[j]];
@@ -1076,11 +1110,11 @@ static void set_trio_QS(args_t *args, trio_t *trio, double *pqs[3], int nqs1)
         for (k=0; k<nqs1; k++) dst[k] = phred2log(qs[k]);
     }
 }
-static void set_trio_QS_noisy(args_t *args, trio_t *trio, double *pqs[3], int nqs1, int n_ad)
+static void set_trio_QS_noisy(args_t *args, trio_t *trio, double *pqs[3], int nqs1, int n_ad, pnoise_t *pnoise)
 {
     int32_t *ad_f = NULL, *ad_m = NULL;     // AD in father and mother
 
-    if ( n_ad && !args->pn_abs && !args->pns_abs && !args->pns_frac ) n_ad = 0;    // AD is not required
+    if ( n_ad && !pnoise->abs && !pnoise->abs1 && !pnoise->frac1 ) n_ad = 0;    // AD is not required
     if ( n_ad )
     {
         // Noise tolerance will be applied only for alleles observed in a single parent (a possible mosaic
@@ -1097,18 +1131,18 @@ static void set_trio_QS_noisy(args_t *args, trio_t *trio, double *pqs[3], int nq
         pqs[j] = args->qs3 + j*nqs1;
         double pn = 0, pns = 0;
         double sum_qs = 0, sum_ad = 0;
-        if ( (args->pn_frac || args->pns_frac) && j!=iCHILD )
+        if ( (pnoise->frac || pnoise->frac1) && j!=iCHILD )
         {
             for (k=0; k<nqs1; k++) sum_qs += qs[k];
-            pn  = sum_qs * args->pn_frac;
-            pns = sum_qs * args->pns_frac;
+            pn  = sum_qs * pnoise->frac;
+            pns = sum_qs * pnoise->frac1;
             if ( n_ad )
             {
                 // "absolute" threshold: find the average QS per read from AD and use that
                 // if bigger than the relative threshold
                 for (k=0; k<n_ad; k++) sum_ad += ad[k];
-                if ( pn  < args->pn_abs * sum_qs / sum_ad ) pn = args->pn_abs * sum_qs / sum_ad;
-                if ( pns < args->pns_abs * sum_qs / sum_ad ) pns = args->pns_abs * sum_qs / sum_ad;
+                if ( pn  < pnoise->abs * sum_qs / sum_ad ) pn = pnoise->abs * sum_qs / sum_ad;
+                if ( pns < pnoise->abs1 * sum_qs / sum_ad ) pns = pnoise->abs1 * sum_qs / sum_ad;
             }
         }
         // Reduce QS for all alleles to account for noise
@@ -1140,12 +1174,12 @@ static void set_trio_QS_noisy(args_t *args, trio_t *trio, double *pqs[3], int nq
             double tmp = qs[k];
             if ( max_qs < tmp ) max_qs = tmp;
         }
-        for (k=0; k<nqs1; k++) 
+        for (k=0; k<nqs1; k++)
         {
             // This uses absolute value of parental QS, does not regard the depth. The penalty is way too
             // strict for high-coverage sites, random parental sequencing errors there are very likely.
             // Puts trio-dnm.40.vcf at -24,-17, which is too strict.
-            //      pqs[j][k] = phred2log(qs[k]);   
+            //      pqs[j][k] = phred2log(qs[k]);
 
 
             // Proportions in log space are not well motivated and does not work well.
@@ -1237,7 +1271,7 @@ static void process_record_naive(args_t *args, bcf1_t *rec)
     for (i=0; i<args->ntrio; i++)
     {
         if ( args->filter && !args->trio[i].pass ) continue;
-        
+
         int ignore_father = 0;  // father is irrelevant for male proband on chrX and can have missing GT
         priors_t *priors;
         if ( !is_chrX ) priors = &args->priors;
@@ -1347,6 +1381,8 @@ static void process_record(args_t *args, bcf1_t *rec)
         hts_expand(double,3*nqs1,args->mqs3,args->qs3);
     }
 
+    pnoise_t *pnoise = (bcf_get_variant_types(rec) & VCF_INDEL) ? &args->pn_indel : &args->pn_snv;
+
     int is_chrX = 0;
     if ( regidx_overlap(args->chrX_idx,bcf_seqname(args->hdr,rec),rec->pos,rec->pos+rec->rlen,NULL) ) is_chrX = 1;
 
@@ -1356,6 +1392,7 @@ static void process_record(args_t *args, bcf1_t *rec)
     else
         for (i=0; i<nsmpl; i++) args->dnm_qual_int[i] = bcf_int32_missing;
     for (i=0; i<nsmpl; i++) args->dnm_allele[i] = bcf_int32_missing;
+    for (i=0; i<nsmpl; i++) args->vaf[i] = bcf_int32_missing;
     for (i=0; i<args->ntrio; i++)
     {
         if ( args->filter && !args->trio[i].pass ) continue;
@@ -1366,7 +1403,7 @@ static void process_record(args_t *args, bcf1_t *rec)
 
         double *pqs[3];
         if ( args->use_model==USE_ACM )
-            set_trio_QS_noisy(args,&args->trio[i],pqs,nqs1,n_ad);
+            set_trio_QS_noisy(args,&args->trio[i],pqs,nqs1,n_ad,pnoise);
         else if ( rec->n_allele > 4 )       // DNG does not use QS, but QS is needed when trimming ALTs
             set_trio_QS(args,&args->trio[i],pqs,nqs1);
 
@@ -1459,26 +1496,34 @@ static void set_option(args_t *args, char *optarg)
     else if ( !strcasecmp(opt,"pn") || !strcasecmp(opt,"pnoise") )
     {
         if ( !val ) error("Error: expected value with -u %s, e.g. -u %s=0.05\n",opt,opt);
-        args->pn_frac = strtod(val,&tmp);
+        double pn_frac,pn_abs = 0;
+        pn_frac = strtod(val,&tmp);
         if ( *tmp && *tmp==',' )
         {
-            args->pn_abs = strtod(tmp+1,&tmp);
+            pn_abs = strtod(tmp+1,&tmp);
             if ( *tmp ) error("Could not parse: -u %s\n", optarg);
         }
-        if ( args->pn_frac<0 || args->pn_frac>1 ) error("Error: expected value from the interval [0,1] for -u %s\n", optarg);
-        if ( args->pn_abs<0 ) error("Error: expected positive value for -u %s\n", optarg);
+        if ( pn_frac<0 || pn_frac>1 ) error("Error: expected value from the interval [0,1] for -u %s\n", optarg);
+        if ( pn_abs<0 ) error("Error: expected positive value for -u %s\n", optarg);
+        args->pn_snv.frac = pn_frac;
+        args->pn_snv.abs  = pn_abs;
+        args->pn_indel = args->pn_snv;
     }
     else if ( !strcasecmp(opt,"pns") )
     {
         if ( !val ) error("Error: expected value with -u %s, e.g. -u %s=0.05\n",opt,opt);
-        args->pns_frac = strtod(val,&tmp);
+        double pn_frac,pn_abs = 0;
+        pn_frac = strtod(val,&tmp);
         if ( *tmp && *tmp==',' )
         {
-            args->pns_abs = strtod(tmp+1,&tmp);
+            pn_abs = strtod(tmp+1,&tmp);
             if ( *tmp ) error("Could not parse: -u %s\n", optarg);
         }
-        if ( args->pns_frac<0 || args->pn_frac>1 ) error("Error: expected value from the interval [0,1] for -u %s\n", optarg);
-        if ( args->pns_abs<0 ) error("Error: expected positive value for -u %s\n", optarg);
+        if ( pn_frac<0 || pn_frac>1 ) error("Error: expected value from the interval [0,1] for -u %s\n", optarg);
+        if ( pn_abs<0 ) error("Error: expected positive value for -u %s\n", optarg);
+        args->pn_snv.frac1 = pn_frac;
+        args->pn_snv.abs1  = pn_abs;
+        args->pn_indel = args->pn_snv;
     }
     else if ( !strcasecmp(opt,"DNG") ) { args->use_model = USE_DNG; args->use_dng_priors = 1; }
     else if ( !strcasecmp(opt,"dng-priors") ) args->use_dng_priors = 1;
@@ -1512,10 +1557,10 @@ int run(int argc, char **argv)
     args->dnm_vaf_tag    = strdup("VAF");
     args->dnm_allele_tag = strdup("VA");
     args->mrate = 1e-8;
-    args->pn_frac   = 0.005;
-    args->pn_abs    = 0;
-    args->pns_frac  = 0.045;
-    args->pns_abs   = 0;
+    args->pn_snv.frac  = 0.005;
+    args->pn_snv.frac1 = 0.045;
+    args->pn_snv.abs   = 0;
+    args->pn_snv.abs1  = 0;
     args->record_cmd_line = 1;
     args->regions_overlap = 1;
     args->targets_overlap = 0;
@@ -1541,6 +1586,7 @@ int run(int argc, char **argv)
         {"with-pad",no_argument,0,13},
         {"chrX",required_argument,0,'X'},
         {"min-score",required_argument,0,'m'},
+        {"strictly-novel",no_argument,0,'n'},
         {"include",required_argument,0,'i'},
         {"exclude",required_argument,0,'e'},
         {"output",required_argument,NULL,'o'},
@@ -1553,13 +1599,15 @@ int run(int argc, char **argv)
         {"targets",1,0,'t'},
         {"targets-file",1,0,'T'},
         {"targets-overlap",required_argument,NULL,15},
+        {"write-index",no_argument,NULL,16},
         {NULL,0,NULL,0}
     };
     int c;
     char *tmp;
-    while ((c = getopt_long(argc, argv, "p:P:o:O:s:i:e:r:R:t:T:m:au:X:",loptions,NULL)) >= 0)
+    double pn_abs, pn_frac;
+    while ((c = getopt_long(argc, argv, "p:P:o:O:s:i:e:r:R:t:T:m:au:X:n",loptions,NULL)) >= 0)
     {
-        switch (c) 
+        switch (c)
         {
             case  1 : args->force_ad = 1; break;
             case  2 : free(args->dnm_score_tag); args->dnm_score_tag = strdup(optarg); break;
@@ -1568,24 +1616,64 @@ int run(int argc, char **argv)
             case  5 : args->use_dng_priors = 1; break;
             case  6 : args->mrate = strtod(optarg,&tmp); if ( *tmp ) error("Could not parse: --mrate %s\n", optarg); break;
             case  7 :
-                args->pn_frac = strtod(optarg,&tmp);
+                pn_frac = strtod(optarg,&tmp);
+                pn_abs  = 0;
                 if ( *tmp && *tmp==',' )
                 {
-                    args->pn_abs = strtod(tmp+1,&tmp);
-                    if ( *tmp ) error("Could not parse: --pn %s\n", optarg);
+                    pn_abs = strtod(tmp+1,&tmp);
+                    if ( *tmp )
+                    {
+                        if ( *tmp!=':' ) error("Could not parse: --pn %s\n", optarg);
+                        if ( !strcasecmp("snv",tmp+1) ) { args->pn_snv.frac = pn_frac; args->pn_snv.abs = pn_abs; }
+                        else if ( !strcasecmp("indel",tmp+1) ) { args->pn_indel.frac = pn_frac; args->pn_indel.abs = pn_abs; }
+                        else error("Could not parse: --pn %s\n", optarg);
+                    }
+                    else
+                    {
+                        args->pn_snv.frac   = pn_frac;
+                        args->pn_snv.abs    = pn_abs;
+                        args->pn_indel.frac = pn_frac;
+                        args->pn_indel.abs  = pn_abs;
+                    }
                 }
-                if ( args->pn_frac<0 || args->pn_frac>1 ) error("Error: expected value from the interval [0,1] for --pn %s\n", optarg);
-                if ( args->pn_abs<0 ) error("Error: expected positive value for --pn %s\n", optarg);
+                else if ( *tmp ) error("Could not parse: --pn %s\n", optarg);
+                else
+                {
+                    args->pn_snv.frac   = pn_frac;
+                    args->pn_indel.frac = pn_frac;
+                }
+                if ( pn_frac<0 || pn_frac>1 ) error("Error: expected value from the interval [0,1] for --pn %s\n", optarg);
+                if ( pn_abs<0 ) error("Error: expected positive value for --pn %s\n", optarg);
                 break;
             case 8 :
-                args->pns_frac = strtod(optarg,&tmp);
+                pn_frac = strtod(optarg,&tmp);
+                pn_abs  = 0;
                 if ( *tmp && *tmp==',' )
                 {
-                    args->pns_abs = strtod(tmp+1,&tmp);
-                    if ( *tmp ) error("Could not parse: --pns %s\n", optarg);
+                    pn_abs = strtod(tmp+1,&tmp);
+                    if ( *tmp )
+                    {
+                        if ( *tmp!=':' ) error("Could not parse: --pns %s\n", optarg);
+                        if ( !strcasecmp("snv",tmp+1) ) { args->pn_snv.frac1 = pn_frac; args->pn_snv.abs1 = pn_abs; }
+                        else if ( !strcasecmp("indel",tmp+1) ) { args->pn_indel.frac1 = pn_frac; args->pn_indel.abs1 = pn_abs; }
+                        else error("Could not parse: --pns %s\n", optarg);
+                    }
+                    else
+                    {
+                        args->pn_snv.frac1   = pn_frac;
+                        args->pn_snv.abs1    = pn_abs;
+                        args->pn_indel.frac1 = pn_frac;
+                        args->pn_indel.abs1  = pn_abs;
+                    }
                 }
-                if ( args->pns_frac<0 || args->pns_frac>1 ) error("Error: expected value from the interval [0,1] for --pns %s\n", optarg);
-                if ( args->pns_abs<0 ) error("Error: expected positive value for --pns %s\n", optarg);
+                else if ( *tmp ) error("Could not parse: --pns %s\n", optarg);
+                else
+                {
+                    args->pn_snv.frac1   = pn_frac;
+                    args->pn_indel.frac1 = pn_frac;
+                }
+                if ( pn_frac<0 || pn_frac>1 ) error("Error: expected value from the interval [0,1] for --pns %s\n", optarg);
+                if ( pn_abs<0 ) error("Error: expected positive value for --pns %s\n", optarg);
                 break;
             case 9  : args->use_model = USE_DNG; args->use_dng_priors = 1; break;
             case 10 : args->with_ppl = 1; break;
@@ -1600,6 +1688,7 @@ int run(int argc, char **argv)
                 args->targets_overlap = parse_overlap_option(optarg);
                 if ( args->targets_overlap < 0 ) error("Could not parse: --targets-overlap %s\n",optarg);
                 break;
+            case 16 : args->write_index = 1; break;
             case 'X': args->chrX_list_str = optarg; break;
             case 'u': set_option(args,optarg); break;
             case 'e':
@@ -1636,6 +1725,7 @@ int run(int argc, char **argv)
             case 'm': args->min_score = strtod(optarg,&tmp);
                       if ( *tmp ) error("Could not parse: -M, --min-score %s\n", optarg);
                       break;
+            case 'n': args->strictly_novel = 1; break;
             case 'h':
             case '?':
             default: error("%s", usage_text()); break;
