@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2022 Genome Research Ltd.
+    Copyright (C) 2022-2023 Genome Research Ltd.
 
     Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -63,6 +63,8 @@ typedef struct
     bcf_hdr_t *hdr;
     bcf_srs_t *sr;
     vcfbuf_t *buf;
+    char *index_fn;
+    int write_index;
 }
 args_t;
 
@@ -91,6 +93,7 @@ static const char *usage_text(void)
         "   -t, --targets REGION             Similar to -r but streams rather than index-jumps\n"
         "   -T, --targets-file FILE          Similar to -R but streams rather than index-jumps\n"
         "       --targets-overlap 0|1|2      Include if POS in the region (0), record overlaps (1), variant overlaps (2) [0]\n"
+        "       --write-index                Automatically index the output files [off]\n"
         "Examples:\n"
         "   bcftools +variant-distance input.bcf -Ob -o output.bcf\n"
         "\n";
@@ -126,6 +129,7 @@ static void init_data(args_t *args)
     if ( args->out_fh == NULL ) error("Can't write to \"%s\": %s\n", args->output_fname, strerror(errno));
 
     if ( bcf_hdr_write(args->out_fh, args->hdr)!=0 ) error("[%s] Error: cannot write to %s\n", __func__,args->output_fname);
+    if ( args->write_index && init_index(args->out_fh,args->hdr,args->output_fname,&args->index_fn)<0 ) error("Error: failed to initialise index for %s\n",args->output_fname);
 
     args->buf = vcfbuf_init(args->hdr, 0);
     vcfbuf_set_opt(args->buf,int,VCFBUF_DUMMY,1)
@@ -134,6 +138,15 @@ static void destroy_data(args_t *args)
 {
     if ( args->filter )
         filter_destroy(args->filter);
+    if ( args->write_index )
+    {
+        if ( bcf_idx_save(args->out_fh)<0 )
+        {
+            if ( hts_close(args->out_fh)!=0 ) error("Error: close failed .. %s\n", args->output_fname?args->output_fname:"stdout");
+            error("Error: cannot write to index %s\n", args->index_fn);
+        }
+        free(args->index_fn);
+    }
     if ( hts_close(args->out_fh)!=0 ) error("[%s] Error: close failed .. %s\n", __func__,args->output_fname);
     bcf_sr_destroy(args->sr);
     vcfbuf_destroy(args->buf);
@@ -233,6 +246,7 @@ int run(int argc, char **argv)
         {"targets-overlap",required_argument,NULL,2},
         {"output",required_argument,NULL,'o'},
         {"output-type",required_argument,NULL,'O'},
+        {"write-index",no_argument,NULL,4},
         {NULL,0,NULL,0}
     };
     int c;
@@ -286,6 +300,7 @@ int run(int argc, char **argv)
                           if ( *tmp || args->clevel<0 || args->clevel>9 ) error("Could not parse argument: --compression-level %s\n", optarg+1);
                       }
                       break;
+            case  4 : args->write_index = 1; break;
             case 'h':
             case '?':
             default: error("%s", usage_text()); break;
